@@ -669,8 +669,12 @@ exo_helper_database_set_default (ExoHelperDatabase *database,
                                  ExoHelper         *helper,
                                  GError           **error)
 {
-  XfceRc *rc;
-  gchar  *key;
+  XfceRc       *rc, *desktop_file;
+  gchar        *key;
+  const gchar  *filename;
+  gchar       **mimetypes;
+  guint         i;
+  gchar        *path;
 
   g_return_val_if_fail (category < EXO_HELPER_N_CATEGORIES, FALSE);
   g_return_val_if_fail (EXO_IS_HELPER_DATABASE (database), FALSE);
@@ -681,7 +685,7 @@ exo_helper_database_set_default (ExoHelperDatabase *database,
   rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, "xfce4/helpers.rc", FALSE);
   if (G_UNLIKELY (rc == NULL))
     {
-      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_IO, _("Failed to open helpers.rc for writing"));
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_IO, _("Failed to open %s for writing"), "helpers.rc");
       return FALSE;
     }
 
@@ -691,6 +695,58 @@ exo_helper_database_set_default (ExoHelperDatabase *database,
   xfce_rc_close (rc);
   g_free (key);
 
+  /* get the desktop filename */
+  switch (category)
+    {
+      case EXO_HELPER_WEBBROWSER:
+        filename = "exo-web-browser.desktop";
+        break;
+
+      case EXO_HELPER_MAILREADER:
+        filename = "exo-mail-reader.desktop";
+        break;
+
+      case EXO_HELPER_FILEMANAGER:
+        filename = "exo-file-manager.desktop";
+        break;
+
+      default:
+        /* no mimetype support for terminals */
+        return TRUE;
+    }
+
+  /* open the mimeapp.list file to set the default handler of the mime type */
+  rc = xfce_rc_config_open (XFCE_RESOURCE_DATA, "applications/mimeapps.list", FALSE);
+  if (G_UNLIKELY (rc == NULL))
+    {
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_IO, _("Failed to open %s for writing"), "mimeapps.list");
+      return FALSE;
+    }
+
+  /* open the exo desktop file to read the mimetypes the file supports */
+  path = g_build_filename ("applications", filename, NULL);
+  desktop_file = xfce_rc_config_open (XFCE_RESOURCE_DATA, path, TRUE);
+  g_free (path);
+
+  if (G_UNLIKELY (desktop_file != NULL))
+    {
+      xfce_rc_set_group (desktop_file, "Desktop Entry");
+      mimetypes = xfce_rc_read_list_entry (desktop_file, "MimeType", ";");
+      if (mimetypes != NULL)
+        {
+          xfce_rc_set_group (rc, "Added Associations");
+
+          for (i = 0; mimetypes[i] != NULL; i++)
+            if (!exo_str_is_empty (mimetypes[i]))
+              xfce_rc_write_entry (rc, mimetypes[i], filename);
+          g_strfreev (mimetypes);
+        }
+
+      xfce_rc_close (desktop_file);
+    }
+
+  xfce_rc_close (rc);
+
   return TRUE;
 }
 
